<?php declare(strict_types=1);

namespace MyPrayerJournal\Domain;

use BitBadger\PDODocument\DocumentException;

/**
 * A note entered on a prayer request
 */
class Note
{
    /**
     * @param string $asOf The date/time this note was recorded
     * @param string $text The text of the note
     */
    public function __construct(public string $asOf, public string $text) { }

    /**
     * Retrieve notes for a given request
     *
     * @param string $id The ID of the request for which notes should be retrieved
     * @return array|Note[] The notes for the request, or an empty array if the request was not found
     * @throws DocumentException If any is encountered
     */
    public static function byRequestId(string $id): array
    {
        $req = Request::byId($id);
        return $req ? $req->notes : [];
    }
}
