<?php declare(strict_types=1);

use BitBadger\PDODocument\{Document, Patch, RemoveFields};
use MyPrayerJournal\{Auth, Table};
use MyPrayerJournal\Domain\{History, Recurrence, RecurrencePeriod, Request, RequestAction};

require '../../start.php';
if ($_SERVER['REQUEST_METHOD'] <> 'POST' && $_SERVER['REQUEST_METHOD'] <> 'PATCH') not_found();

Auth::requireUser(false);

$now        = new DateTimeImmutable('now');
$recurrence = new Recurrence(RecurrencePeriod::from($_POST['recurType'] ?? $_PATCH['recurType']));
if ($recurrence->period <> RecurrencePeriod::Immediate) {
    $recurrence->interval = (int)($_POST['recurCount'] ?? $_PATCH['recurCount']);
}

switch ($_SERVER['REQUEST_METHOD']) {
    case 'POST':
        Document::insert(Table::REQUEST, new Request(
            enteredOn:  $now->format('c'),
            userId:     $_SESSION['user_id'],
            recurrence: $recurrence,
            history:    [new History($now->format('c'), RequestAction::Created, $_POST['requestText'])]));
        //Messages.pushSuccess ctx "Added prayer request" "/journal"
        see_other('/journal');

    case 'PATCH':
        $req = Request::byId($_PATCH['requestId']);
        if (!$req) not_found();
        $patch = [];
        // update recurrence if changed
        if ($recurrence != $req->recurrence) {
            $patch['recurrence'] = $recurrence;
            if ($recurrence->period == RecurrencePeriod::Immediate) {
                RemoveFields::byId(Table::REQUEST, $req->id, ['showAfter']);
            }
        }
        // append history
        $upd8Text = trim($_PATCH['requestText']);
        $text     = $upd8Text == '' || $upd8Text == $req->currentText() ? null : $upd8Text;
        array_unshift($req->history, new History($now->format('c'), RequestAction::from($_PATCH['status']), $text));
        $patch['history'] = $req->history;
        Patch::byId(Table::REQUEST, $req->id, $patch);
        //Messages.pushSuccess ctx "Prayer request updated successfully" nextUrl
        see_other($_PATCH['returnTo']);
}
